@@ -53,7 +53,7 @@ class Node(object):
     def updateDrivers(self, drivers):
         self.drivers = deepcopy(drivers)
 
-    def _updateDrivers(self, poly, address):
+    def _updateDrivers(self, poly, address, init=True):
         db_drivers = poly.db_getNodeDrivers(address)
         try:
             for drv in db_drivers:
@@ -144,6 +144,7 @@ class Node(object):
         }
 
         self.poly.send(message, 'command')
+        self.name = newname
 
     def reportCmd(self, command, value=None, uom=None):
         message = {
@@ -164,13 +165,13 @@ class Node(object):
                 fun = self.commands[command['cmd']]
                 fun(self, command)
             else:
-                NLOGGER.error('command {} not defined'.format(command['cmd']))
+                NLOGGER.error('node {} command {} not defined'.format(self.address,command['cmd']))
         elif 'success' in command:
             if not command['success']:
-                NLOGGER.error('Command message failed: {}'.format(command))
+                NLOGGER.error('Command message failed for node {}: {}'.format(self.address,command))
 
         else:
-            NLOGGER.error('Invalid command message: {}'.format(command))
+            NLOGGER.error('Invalid command message for node {}: {}'.format(self.address,command))
 
 
     def start(self):
